# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Copyright (c) 2007 PloneGov
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = '''Gauthier BASTIEN <gbastien@commune.sambreville.be>'''
__docformat__ = 'plaintext'

'''PloneMeeting exportimport setup.
   This is the exportimport file for GenericSetup. See configure.zcml and
   profiles/default for more information.'''

# ------------------------------------------------------------------------------
import os.path
from Products.CMFCore.utils import getToolByName
from Products.PlacelessTranslationService import utranslate
from Products.CMFCore.permissions import View
from Products.CMFCore.permissions import AccessContentsInformation
from Products.PloneMeeting import PloneMeetingError
from zExceptions import BadRequest
import transaction
from Products.PloneMeeting.config import *
import logging
logger = logging.getLogger('PloneMeeting')

# PloneMeeting-Error related constants -----------------------------------------
MEETING_CONFIG_ERROR = 'A validation error occurred while instantiating ' \
                       'meeting configuration with id "%s". %s'
MEETING_CONFIG_FIELD_ERROR = 'Validation error on field "%s" of meeting ' \
                             'config "%s". %s'
MEETING_ID_EXISTS = 'The meeting config with id "%s" already exists.'

# ------------------------------------------------------------------------------
class ToolInitializer:
    '''Initializes the PloneMeeting tool based on information from a given
       PloneMeeting profile.'''
    successMessage = "The PloneMeeting tool has been successfully initialized."
    fieldsWithCustomValidators = (
        'itemConditionsInterface', 'itemActionsInterface',
        'meetingConditionsInterface', 'meetingActionsInterface',
        'adviceConditionsInterface', 'adviceActionsInterface')

    def __init__(self, context, productname):
        site = context.getSite()
        self.context = context
        self.productname = productname
        self.tool = site.portal_plonemeeting
        self.wfTool = site.portal_workflow
        self.registerTool = site.portal_registration
        self.sourceUsers = site.acl_users.source_users
        self.groupsTool = site.portal_groups
        self.profileData = self.getProfileData(context)
        # Initialize the tool
        self.tool.setMeetingFolderTitle(self.profileData.meetingFolderTitle)
        self.tool.setUnoEnabledPython(self.profileData.unoEnabledPython)
        self.tool.setOpenOfficePort(self.profileData.openOfficePort)
        self.tool.setPloneDiskAware(self.profileData.ploneDiskAware)
        self.tool.setNavigateLocally(self.profileData.navigateLocally)
        self.tool.setFunctionalAdminEmail(self.profileData.functionalAdminEmail)
        self.tool.setFunctionalAdminName(self.profileData.functionalAdminName)
        self.tool.setUsedColorSystem(self.profileData.usedColorSystem)
        self.tool.setColorSystemDisabledFor(
            self.profileData.colorSystemDisabledFor)
        self.tool.setRestrictUsers(self.profileData.restrictUsers)
        self.tool.setUnrestrictedUsers(self.profileData.unrestrictedUsers)
        self.createUsersAndGroups()
        self.createExternalApplications()

    def createExternalApplications(self):
        for extApp in self.profileData.externalApplications:
            self.tool.invokeFactory('ExternalApplication', **extApp.__dict__)

    def createUser(self, userDescr):
        '''Creates user described by p_userDescr if it does not already exist.
        '''
        if userDescr.id not in self.sourceUsers.getUserNames():
            self.registerTool.addMember(
                userDescr.id, userDescr.password,
                ['Member'] + userDescr.globalRoles,
                properties={'username': userDescr.id, 'email': userDescr.email,
                            'fullname': userDescr.fullname or ''})
            # Add the user to some Plone groups
            for groupDescr in userDescr.ploneGroups:
                # Create the group if it does not exist
                if not self.groupsTool.getGroupById(groupDescr.id):
                    self.groupsTool.addGroup(groupDescr.id,
                                             title=groupDescr.title)
                    self.groupsTool.setRolesForGroup(groupDescr.id,
                                                     groupDescr.roles)
                self.groupsTool.addPrincipalToGroup(userDescr.id,
                                                    groupDescr.id)


    def createUsersAndGroups(self):
        '''Loads, from the current profile, MeetingGroups and potentially create
           a bunch of test users in those groups.'''
        for groupDescr in self.profileData.groups:
            self.tool.invokeFactory('MeetingGroup', **groupDescr.getData())
            group = getattr(self.tool, groupDescr.id)
            group._at_creation_flag = False
            # See note on _at_creation_flag attr below.
            group.at_post_create_script()
            # Create users
            for userDescr in groupDescr.getUsers():
                self.createUser(userDescr)
            # Add users in the correct Plone groups.
            for groupSuffix in MEETINGROLES.iterkeys():
                groupId = group.getPloneGroupId(groupSuffix)
                for userDescr in getattr(groupDescr, groupSuffix):
                    if userDescr.id not in \
                       self.groupsTool.getGroupMembers(groupId):
                        self.groupsTool.addPrincipalToGroup(
                            userDescr.id, groupId)
            if not groupDescr.active:
                self.wfTool.doActionFor(group, 'deactivate')
        # Create users that are outside any PloneMeeting group (like WebDAV
        # users)
        for userDescr in self.profileData.usersOutsideGroups:
            self.createUser(userDescr)

    def getProfileData(self, context):
        '''Loads, from the current profile, the data to import into the tool:
        meeting config(s), categories, etc.'''
        pp = context._profile_path
        profileModule = pp[pp.rfind(self.productname):].replace('/', '.')
        profileModule = profileModule.replace('\\', '.')
        exec 'from Products.%s.import_data import data' % profileModule
        return data

    def createMeetingConfig(self, mConfig):
        '''Creates, within the tool, the MeetingConfig instances based on data
           in p_mConfig.'''
        self.tool.invokeFactory(
            'MeetingConfig', **mConfig.getData(self.context._profile_path))
        meetingConfig = getattr(self.tool, mConfig.id)
        # Validates meeting config (validation seems not triggered
        # automatically when an object is created from code).
        errors = meetingConfig.schema.validate(meetingConfig)
        if errors:
            raise PloneMeetingError(
                MEETING_CONFIG_ERROR % meetingConfig.getId(), errors)
        # Goddamned, custom validators do no seem to be called.
        for fieldName in self.fieldsWithCustomValidators:
            exec 'fieldValue = meetingConfig.get%s%s()' % (
                fieldName[0].upper(), fieldName[1:])
            for validator in meetingConfig.getField(fieldName).validators:
                res = validator[0](fieldValue, instance=meetingConfig)
                if not isinstance(res, bool):
                    raise PloneMeetingError(
                        MEETING_CONFIG_FIELD_ERROR % (
                            fieldName, meetingConfig.getId(), res))
        # Goddamned end.
        meetingConfig._at_creation_flag = False # It seems that this flag,
        # internal to Archetypes, is not set when the meeting config is
        # created from a profile, not through-the-web. So we force it here.
        # This way, once we will update the meeting config through-the-web,
        # at_post_create_script will not be called again, but
        # at_post_edit_script instead.
        meetingConfig.at_post_create_script() # Is called automatically only
        # when the object is created through-the-web.
        if not mConfig.active:
            self.wfTool.doActionFor(meetingConfig, 'deactivate')
        return meetingConfig

    def addCategories(self, meetingConfig, mConfig, classifiers=False):
        '''Creates the categories or classifiers for this meeting config.'''
        if classifiers:
            folderName = TOOL_FOLDER_CLASSIFIERS
            categs = mConfig.classifiers
        else:
            folderName = TOOL_FOLDER_CATEGORIES
            categs = mConfig.categories
        categoriesFolder = getattr(meetingConfig, folderName)
        for catDescr in categs:
            categoriesFolder.invokeFactory('MeetingCategory', catDescr.id,
                                            title=catDescr.title,
                                            description=catDescr.description)
            if not catDescr.active:
                cat = getattr(categoriesFolder, catDescr.id)
                self.wfTool.doActionFor(cat, 'deactivate')

    def addRecurringItems(self, meetingConfig, mConfig):
        '''Creates the recurring items to be inserted in each meeting.'''
        recFolder = getattr(meetingConfig, TOOL_FOLDER_RECURRING_ITEMS)
        for recItemDescr in mConfig.recurringItems:
            recFolder.invokeFactory(meetingConfig.getItemTypeName(),
                                    **recItemDescr.__dict__)
            recItem = getattr(recFolder, recItemDescr.id)
            recItem.at_post_create_script()

    def addFileTypes(self, meetingConfig, mConfig):
        '''Adds the meeting file types.'''
        fileTypesFolder = getattr(meetingConfig, TOOL_FOLDER_FILE_TYPES)
        for ft in mConfig.meetingFileTypes:
            iconPath = '%s/images/%s' % (self.context._profile_path,ft.iconFile)
            iconFile = file(iconPath, 'rb')
            iconContent = iconFile.read()
            fileTypesFolder.invokeFactory(
                'MeetingFileType', ft.id, title=ft.title,
                theIcon=iconContent, predefinedTitle=ft.predefinedFileTitle,
                decisionRelated=ft.decisionRelated)
            iconFile.close()
            if not ft.active:
                fileType = getattr(fileTypesFolder, ft.id)
                self.wfTool.doActionFor(fileType, 'deactivate')

    def addAgreementLevels(self, meetingConfig, mConfig):
        '''Adds the meeting file types.'''
        agreementLevelsFolder = getattr(meetingConfig, TOOL_FOLDER_AGREEMENT_LEVELS)
        for maal in mConfig.agreementLevels:
            iconPath = '%s/images/%s' % (self.context._profile_path,maal.iconFile)
            iconFile = file(iconPath, 'rb')
            iconContent = iconFile.read()
            agreementLevelsFolder.invokeFactory(
                'MeetingAdviceAgreementLevel', maal.id, title=maal.title,
                theIcon=iconContent)
            iconFile.close()
            if not maal.active:
                a_level = getattr(agreementLevelsFolder, maal.id)
                self.wfTool.doActionFor(a_level, 'deactivate')

    def addPodTemplates(self, meetingConfig, mConfig):
        podTemplatesFolder = getattr(meetingConfig, TOOL_FOLDER_POD_TEMPLATES)
        for podTemplateDescr in mConfig.podTemplates:
            fileName = podTemplateDescr.podTemplate
            filePath = '%s/templates/%s' % (self.context._profile_path,fileName)
            podTemplateDescr.podTemplate = file(filePath)
            podTemplatesFolder.invokeFactory('PodTemplate',
                                             **podTemplateDescr.__dict__)
            podTemplateDescr.podTemplate.close()
            podTemplateDescr.podTemplate = fileName
            podTemplate = getattr(podTemplatesFolder, podTemplateDescr.id)
            if not podTemplateDescr.active:
                self.wfTool.doActionFor(podTemplate, 'deactivate')

    def run(self):
        for mConfig in self.profileData.meetingConfigs:
            try:
                meetingConfig = self.createMeetingConfig(mConfig)
                self.addCategories(meetingConfig, mConfig)
                self.addCategories(meetingConfig, mConfig, classifiers=True)
                self.addRecurringItems(meetingConfig, mConfig)
                self.addFileTypes(meetingConfig, mConfig)
                self.addAgreementLevels(meetingConfig, mConfig)
                self.addPodTemplates(meetingConfig, mConfig)
            except BadRequest:
                # If we raise a BadRequest, it is that the id is already in use.
                logger.warn(MEETING_ID_EXISTS % mConfig.id)
        return self.successMessage

# Functions that correspond to the PloneMeeting profile import steps -----------
def installProducts(context):
    '''Installs the necessary products for running PloneMeeting.'''
    portal = context.getSite()
    qi = getToolByName(portal, 'portal_quickinstaller')
    if not qi.isProductInstalled('Archetypes'):
        qi.installProduct('Archetypes')
    if not qi.isProductInstalled('PloneMeeting'):
        qi.installProduct('PloneMeeting')
    transaction.commit()
    return "Necessary products for PloneMeeting installed."

def initializeTool(context):
    '''Initialises the PloneMeeting tool based on information from the current
       profile.'''
    return ToolInitializer(context, PROJECTNAME).run()
# ------------------------------------------------------------------------------
